use std::f32::consts::PI;

use bevy::{
	input::gamepad::{GamepadConnection, GamepadEvent},
	prelude::*,
	render::view::VisibilityPlugin,
};

use crate::{
	common::{
		animation::{AnimationBundle, AnimationIndices, AnimationPlugin, AnimationTimer},
		velocity::Velocity,
	},
	gamepad::GamepadOne,
	level::{Collider, UnitBundle},
	GameState, SCREEN_HEIGHT, SCREEN_WIDTH, START_POSITION,
};

pub struct PlayerPlugin;

impl Plugin for PlayerPlugin {
	fn build(&self, app: &mut App) {
		app.add_systems(Startup, setup);
		app.add_systems(
			FixedUpdate,
			(move_player).run_if(in_state(GameState::Playing)),
		);
	}
}

#[derive(Component)]
pub struct Player;

fn setup(
	mut commands: Commands,
	asset_server: Res<AssetServer>,
	mut texture_atlas_layouts: ResMut<Assets<TextureAtlasLayout>>,
) {
	let layout = TextureAtlasLayout::from_grid(UVec2::splat(135), 2, 1, None, None);
	let texture_atlas_layouts = texture_atlas_layouts.add(layout);

	commands.spawn((
		Player,
		Collider,
		UnitBundle {
			sprite: SpriteBundle {
				transform: Transform {
					translation: START_POSITION,
					scale: Vec2::new(0.5, 0.5).extend(1.0),
					..Default::default()
				},
				visibility: Visibility::Hidden,
				texture: asset_server.load("player.png"),
				..Default::default()
			},
			animator: AnimationBundle {
				atlas: TextureAtlas {
					layout: texture_atlas_layouts,
					index: 0,
				},
				timer: AnimationTimer(Timer::from_seconds(0.3, TimerMode::Repeating)),
				indices: AnimationIndices {
					first_index: 0,
					last_index: 1,
				},
			},
			..Default::default()
		},
	));
}

fn move_player(
	keys: Res<ButtonInput<KeyCode>>,
	axes: Res<Axis<GamepadAxis>>,
	gamepads: Res<ButtonInput<GamepadButton>>,
	gamepad1: Option<Res<GamepadOne>>,
	mut query: Query<(&mut Transform, &mut Velocity), With<Player>>,
	window: Query<&Window>,
) {
	let (mut transform, mut velocity) = query.single_mut();
	velocity.0 = Vec2::ZERO;

	if let Some(gamepad) = gamepad1.as_deref() {
		handle_gamepad_movement(&axes, gamepad, &gamepads, &mut velocity);
	};

	if keys.pressed(KeyCode::ArrowRight) {
		velocity.x += 1.0;
	}
	if keys.pressed(KeyCode::ArrowLeft) {
		velocity.x -= 1.0;
	}
	if keys.pressed(KeyCode::ArrowUp) {
		velocity.y += 1.0;
	}
	if keys.pressed(KeyCode::ArrowDown) {
		velocity.y -= 1.0;
	}

	if velocity.length() > 0.0 {
		velocity.0 = velocity.normalize() * 400.0;
		let angle = velocity.to_angle() - PI / 2.0;
		let angle = Quat::from_rotation_z(angle);
		transform.rotation = angle;
	}

	let window = window.single();
	let screen_size: Vec3 = window.physical_size().as_vec2().extend(1.0);
	transform.translation = transform.translation.clamp(Vec3::ZERO, screen_size);
}

fn handle_gamepad_movement(
	axes: &Axis<GamepadAxis>,
	gamepad: &GamepadOne,
	gamepads: &Res<ButtonInput<GamepadButton>>,
	velocity: &mut Velocity,
) {
	let axis_lx = GamepadAxis {
		gamepad: gamepad.0,
		axis_type: GamepadAxisType::LeftStickX,
	};
	let axis_ly = GamepadAxis {
		gamepad: gamepad.0,
		axis_type: GamepadAxisType::LeftStickY,
	};

	if let (Some(x), Some(y)) = (axes.get(axis_lx), axes.get(axis_ly)) {
		let left_stick = Vec2::new(x, y);
		if left_stick.length() > 0.25 {
			*velocity = Velocity(left_stick);
			return;
		}
	}

	let up_btn = GamepadButton {
		gamepad: gamepad.0,
		button_type: GamepadButtonType::DPadUp,
	};

	let down_btn = GamepadButton {
		gamepad: gamepad.0,
		button_type: GamepadButtonType::DPadDown,
	};

	let left_btn = GamepadButton {
		gamepad: gamepad.0,
		button_type: GamepadButtonType::DPadLeft,
	};

	let right_btn = GamepadButton {
		gamepad: gamepad.0,
		button_type: GamepadButtonType::DPadRight,
	};

	if gamepads.pressed(right_btn) {
		velocity.x += 1.0;
	}
	if gamepads.pressed(left_btn) {
		velocity.x -= 1.0;
	}
	if gamepads.pressed(up_btn) {
		velocity.y += 1.0;
	}
	if gamepads.pressed(down_btn) {
		velocity.y -= 1.0;
	}
}
